package plugins.wlx;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Frame;
import java.awt.GraphicsConfiguration;
import java.awt.Panel;

import javax.swing.JRootPane;
import javax.swing.JScrollPane;
import javax.swing.UIManager;

import org.eclipse.swt.SWT;
import org.eclipse.swt.awt.SWT_AWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;

/**
 * @author Ken
 * 
 */
public abstract class SwingWLXPlugin extends SWTWLXPlugin {

	/**
	 * {@inheritDoc}
	 */
	public void listCloseWindow() {
		// TODO Auto-generated method stub

	}

	/**
	 * {@inheritDoc}
	 */
	public void listLoad(final Shell shell, final String input,
			final int showFlags) {
		Composite composite = new Composite(shell, SWT.NO_BACKGROUND
				| SWT.EMBEDDED);

		/*
		 * Set a Windows specific AWT property that prevents heavyweight
		 * components from erasing their background. Note that this is a
		 * global property and cannot be scoped. It might not be suitable
		 * for your application.
		 */
		try {
			System.setProperty("sun.awt.noerasebackground", "true");
		} catch (NoSuchMethodError error) {
			throw new RuntimeException(error);
		}

		/* Create and setting up frame */
		Frame frame = SWT_AWT.new_Frame(composite);
		UIManager.put("swing.boldMetal", Boolean.FALSE);

		Panel panel = new Panel(new BorderLayout()) {
			public void update(final java.awt.Graphics g) {
				/* Do not erase the background */
				paint(g);
			}
		};
		frame.add(panel);
		JRootPane root = new JRootPane();
		panel.add(root);
		java.awt.Container contentPane = root.getContentPane();
		contentPane.add(new JScrollPane(listLoad(input, showFlags)));
	}

	/**
	 * ListLoad is called when a user opens lister with F3 or the Quick View
	 * Panel with Ctrl+Q, and when the definition string either doesn't exist,
	 * or its evaluation returns true.<BR>
	 * Please note that multiple Lister windows can be open at the same time!
	 * Therefore you cannot save settings in global variables. You can call
	 * RegisterClass with the parameter cbWndExtra to reserve extra space for
	 * your data, which you can then access via GetWindowLong(). Or use an
	 * internal list, and store the list parameter via
	 * SetWindowLong(hwnd,GWL_ID,...). Lister will subclass your window to catch
	 * some hotkeys like 'n' or 'p'.<BR>
	 * When lister is activated, it will set the focus to your window. If your
	 * window contains child windows, then make sure that you set the focus to
	 * the correct child when your main window receives the focus!<BR>
	 * 
	 * If lcp_forceshow is defined, you may try to load the file even if the
	 * plugin wasn't made for it. Example: A plugin with line numbers may only
	 * show the file as such when the user explicitly chooses 'Image/Multimedia'
	 * from the menu.
	 * 
	 * Lister plugins which only create thumbnail images do not need to
	 * implement this function.
	 * 
	 * @param input
	 *            The name of the file which has to be loaded.
	 * @param showFlags
	 *            A combination of the following flags:
	 *            <UL>
	 *            <LI>lcp_wraptext Text: Word wrap mode is checked
	 *            <LI>lcp_fittowindow Images: Fit image to window is checked
	 *            <LI>lcp_ansi Ansi charset is checked
	 *            <LI>lcp_ascii Ascii(DOS) charset is checked
	 *            <LI>lcp_variable Variable width charset is checked
	 *            <LI>lcp_forceshow User chose 'Image/Multimedia' from the
	 *            menu. See remarks.
	 *            </UL>
	 *            You may ignore these parameters if they don't apply to your
	 *            document type.
	 * @return Return a Swing Component if load succeeds, NULL otherwise.
	 *         If NULL is returned, Lister will try the next plugin.
	 * @see WLXPluginInterface#listLoad(int, String, int)
	 */
	public abstract Component listLoad(final String input,
			final int showFlags);

}
